package com.nisum.example.webflux.controller;

import com.nisum.example.webflux.model.Book;
import com.nisum.example.webflux.service.BookService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;

import java.util.Arrays;
import java.util.List;

/**
 * BookController having POST end point for sending data
 * using Request body and Query Param
 */
@RestController
public class BookController {

    @Autowired
    private BookService bookService;

    /**
     * http POST method to perform insertion of records
     * @param books
     * @return Flux<T>
     */
    //sending data as request body
    @PostMapping("/books")
    @ResponseStatus(HttpStatus.CREATED)
    Flux<Book> saveBook(@RequestBody List<Book> books){
        return bookService.insertBook(books);
    }

    /**
     * http POST method to perform insertion of records
     * @param bookId, bookName, bookAuthor, bookPrice
     * @return Flux<T>
     */
    //sending data as queryparam
    @PostMapping("/booksall")
    @ResponseStatus(HttpStatus.CREATED)
    //http://localhost:8080/booksall?bookId=1&bookName=Webflux&bookAuthor=Steve&bookPrice=1500
    Flux<Book> createBook(@RequestParam("bookId") Long bookId,
                          @RequestParam("bookName") String bookName,
                          @RequestParam("bookAuthor") String bookAuthor,
                          @RequestParam("bookPrice") Long bookPrice){
        List<Book> books= Arrays.asList(new Book(bookId,bookName,bookAuthor,bookPrice));
        return bookService.insertBook(books);
    }
}
