package com.nisum.example.webflux.controller;

import com.nisum.example.webflux.model.Book;
import com.nisum.example.webflux.repository.BookRepository;
import com.nisum.example.webflux.service.BookServiceImpl;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.reactive.WebFluxTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.annotation.Import;
import org.springframework.http.MediaType;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.test.web.reactive.server.WebTestClient;
import org.springframework.web.reactive.function.BodyInserters;
import reactor.core.publisher.Flux;

import java.util.Arrays;
import java.util.List;

@ExtendWith(SpringExtension.class)
@WebFluxTest(controllers = BookController.class)
@Import(BookServiceImpl.class)
public class BookControllerTest {

    @MockBean
    BookRepository bookRepository;

    @Autowired
    WebTestClient webTestClient;

    @Test
    void testSaveBook(){
        List<Book> bookList= Arrays
                .asList(new Book(101L,"Spring","James",1500L));
       Flux<Book> bookFlux=Flux.fromIterable(bookList);

        Mockito.when(bookRepository.saveAll(bookList)).thenReturn(bookFlux);

        webTestClient.post()
                .uri("/books")
                .contentType(MediaType.APPLICATION_JSON)
                .body(BodyInserters.fromObject(bookList))
                .exchange()
                .expectStatus().isCreated();

       Mockito.verify(bookRepository,Mockito.times(1)).saveAll(bookList);
    }

    @Test
    void testCreateBook(){
        List<Book> bookList= Arrays
                .asList(new Book(101L,"Spring","James",1500L));
        Flux<Book> bookFlux=Flux.fromIterable(bookList);

        Mockito.when(bookRepository.saveAll(bookList)).thenReturn(bookFlux);

        webTestClient.post()
                .uri(uriBuilder -> uriBuilder.path("/booksall")
                .queryParam("bookId","101")
                .queryParam("bookName","Spring")
                .queryParam("bookAuthor","James")
                .queryParam("bookPrice","1500")
                        .build())
                .exchange()
                .expectStatus().isCreated()
                .expectBodyList(Book.class);
        Mockito.verify(bookRepository,Mockito.times(1)).saveAll(bookList);
    }
}

